/******************************************************************************
 * Away_Command.java - Player says he is away from his computer.
 * $Id: Away_Command.java,v 1.2 2011/05/31 13:02:43 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Away_Command.java,v $
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.gui.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.server.Server;
import net.sourceforge.jibs.util.JibsWriter;

import java.util.Collection;

/** Player says he is away from his computer.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.2 $ <br> $Date: 2011/05/31 13:02:43 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Away_Command.java">cvs Away_Command.java</a>
 */
public class Away_Command implements JibsCommand {
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		JibsWriter out = player.getOutputStream();
		String msg = null;
		String strPlayer = null;
		StringBuffer sBuffer = null;

		if (args.length > 1) {
			player.changeToggle("away", Boolean.TRUE);
			player.setAwayMsg(strArgs);
			// m_you_away=You're away. Please type 'back'
			msg = jibsMessages.convert("m_you_away");
			out.println(msg);
			player.informToggleChange();
		} else {
			Server server = jibsServer.getServer();
			Collection<Player> awayPlayerCollection = server.getAwayPlayer();

			if ((awayPlayerCollection == null) ||
					(awayPlayerCollection.size() == 0)) {
				// m_away_none=None of the users is away.
				msg = jibsMessages.convert("m_away_none");
				out.println(msg);
			} else {
				// m_away_list=The following users are away:
				msg = jibsMessages.convert("m_away_list");
				out.println(msg);

				for (Player awayPlayer : awayPlayerCollection) {
					sBuffer = new StringBuffer();
					strPlayer = awayPlayer.getName();

					if (strPlayer.equals(player.getName())) {
						strPlayer = "You";
					}

					sBuffer.append(strPlayer);
					sBuffer.append(":");
					sBuffer.append(awayPlayer.getAwayMsg());
					out.println(sBuffer.toString());
				}
			}
		}

		return true;
	}
}
