/******************************************************************************
 * Exit_Command.java - Client is leaving.
 * $Id: Exit_Command.java,v 1.4 2011/06/02 19:29:43 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Exit_Command.java,v $
 * Revision 1.4  2011/06/02 19:29:43  dick
 * When a Player exits the server, he is no longer a watcher.
 *
 * Revision 1.3  2011/05/31 13:02:42  dick
 * Bucko headers.
 *
 * Revision 1.2  2011/05/30 04:22:05  dick
 * Read the logout file from the resources by default.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.gui.JibsMessages;
import net.sourceforge.jibs.gui.JibsTextArea;
import net.sourceforge.jibs.server.*;
import net.sourceforge.jibs.util.ClipConstants;
import net.sourceforge.jibs.util.JibsWriter;

import java.io.*;

import java.util.*;
import java.util.Map.Entry;

/** A client has left the building.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.4 $ <br> $Date: 2011/06/02 19:29:43 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Exit_Command.java">cvs Exit_Command.java</a>
 */
public class Exit_Command implements JibsCommand {
	
	public void displayLogoff(JibsServer jibsServer, JibsWriter out) {
		String sName = jibsServer.getResource("logout");
		File file = new File(sName);
		BufferedReader inp = null;
		;

		try {
			if (file.canRead()) {
				inp = new BufferedReader(new FileReader(file));
			} else {
				InputStream is = ClassLoader.getSystemResourceAsStream("conf/logout.txt");
				inp = new BufferedReader(new InputStreamReader(is));
			}


			String theLine = inp.readLine();

			while (theLine != null) {
				if (theLine.equals("")) {
					theLine = " ";
				}

				out.println(theLine);
				theLine = inp.readLine();
			}

			inp.close();
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		} finally {
			try {
				if (inp != null) {
					inp.close();
				}
			} catch (IOException e) {
				jibsServer.logException(e);
			}
		}
	}

	@SuppressWarnings("unchecked")
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		JibsWriter out = player.getOutputStream();
		player.stopWatching();
		try {
			if (out != null) {
				displayLogoff(jibsServer, out);
			}

			JibsGame game = player.getGame();

			if (game != null) {
				if (!game.getBackgammonBoard().isEnded()) {
					game.save();
				}
			}

			ClientWorker cw = player.getClientWorker();

			if (cw != null) {
				player.getClientWorker().getSocket().close();
				player.getClientWorker().stopWatchThread();
				player.getClientWorker().disConnectPlayer(player);
				cw.getSocket().close();
			}

			// m_you_log_out=%0 logs out.
			Object[] obj = new Object[] { player.getName() };
			String msg = jibsMessages.convert("m_you_log_out", obj);
			JibsTextArea.log(jibsServer, msg, true);

			Map omap = (Map) jibsServer.getServer().getAllClients().clone();
			Map map = Collections.synchronizedMap(omap);
			Set<Entry> set = map.entrySet();

			synchronized (map) {
				for (Entry entry : set) {
					Player curPlayer = (Player) entry.getValue();
					msg = ClipConstants.CLIP_LOGOUT + " " + player.getName() +
					" " + msg;
					curPlayer.getOutputStream().println(msg);
				}
			}
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		}

		return false;
	}
}
