/******************************************************************************
 * Move_Command - A client moves his checkers.
 * $Id: Move_Command.java,v 1.5 2011/06/03 19:27:28 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Move_Command.java,v $
 * Revision 1.5  2011/06/03 19:27:28  dick
 * game.winGame[XO]() just needs the points, not the other parameters.
 *
 * Revision 1.4  2011/06/03 00:25:09  dick
 * Player.show2Watcher() becomes show2Watchers().
 *
 * Revision 1.3  2011/06/02 19:31:46  dick
 * show2WatcherMove() is now just show2Watcher().
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.backgammon.MoveBackgammon;
import net.sourceforge.jibs.gui.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsWriter;

/** A client moves his checkers.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.5 $ <br> $Date: 2011/06/03 19:27:28 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Move_Command.java">cvs Move_Command.java</a>
 */
public class Move_Command implements JibsCommand {
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String msg = null;
		JibsGame game = player.getGame();

		if (game != null) {
			BackgammonBoard board = game.getBackgammonBoard();
			Player player1 = board.getPlayerX();
			Player player2 = board.getPlayerO();
			Player opponent = board.getOpponent(player);
			BackgammonBoard bgBoard = game.getBackgammonBoard();
			BackgammonBoard opBoard;
			JibsWriter out1 = player1.getOutputStream();
			JibsWriter out2 = player2.getOutputStream();
			MoveBackgammon move;
			BackgammonBoard movedBoard = null;
			int turn = board.getTurn();
			strArgs = strArgs.trim();

			switch (turn) {
			case 1: // O
			move = new MoveBackgammon(bgBoard);

			if (move.checkMove(player2, strArgs, bgBoard.getCanMove())) {
				movedBoard = move.placeMoveO(strArgs);
				game.setBackgammonBoard(movedBoard);
				opBoard = movedBoard.switch2O();
				opBoard.outBoard(out2, "You", -turn, 0, 0, 0, 0);
				//player2.show2WatcherBoard(opBoard, player2.getName(), 0, 0,
				//		0, 0);		// not neccessary (and fibs doesn't do it).  A board will be along in 7 lines

				// m_both_move=%0 moves %1.
				Object[] obj = new Object[] { player2.getName(), strArgs };
				msg = jibsMessages.convert("m_both_move", obj);
				out1.println(msg);
				player1.show2Watchers(msg);
				player2.show2Watchers(msg);
				movedBoard.outBoard(out1, "You", -turn, 0, 0, 0, 0);
				player1.show2WatcherBoard(movedBoard, player1.getName(), 0,
						0, 0, 0);
			}

			break;

			case -1: // X
				move = new MoveBackgammon(bgBoard);

				if (move.checkMove(player1, strArgs, bgBoard.getCanMove())) {
					movedBoard = move.placeMoveX(strArgs);
					game.setBackgammonBoard(movedBoard);
					movedBoard.outBoard(out1, "You", -turn, 0, 0, 0, 0);
					player1.show2WatcherBoard(movedBoard, player1.getName(), 0,
							0, 0, 0);

					// m_both_move=%0 moves %1.
					Object[] obj = new Object[] { player1.getName(), strArgs };
					msg = jibsMessages.convert("m_both_move", obj);
					out2.println(msg);
					player1.show2Watchers(msg);
					player2.show2Watchers(msg);
					opBoard = movedBoard.switch2O();
					opBoard.outBoard(out2, "You", -turn, 0, 0, 0, 0);
					player1.show2WatcherBoard(opBoard, player1.getName(), 0, 0,
							0, 0);
				}

				break;
			}

			board = game.getBackgammonBoard();

			// won?
			bgBoard = game.getBackgammonBoard();

			if (bgBoard.getOnHome1() >= 15) {
				// Player1 has won !
				game.winGameX(-1);

				return true;
			}

			if (bgBoard.getOnHome2() >= 15) {
				// Player2 has won !
				game.winGameO(-1);

				return true;
			}

			JibsWriter opponentOut = null;

			switch (board.getTurn()) {
			case 1: // O
				board.getoDie1().setValue(0);
				board.getoDie2().setValue(0);

				break;

			case -1: // X
				board.getpDie1().setValue(0);
				board.getpDie2().setValue(0);

				break;
			}

			board.setTurn(-board.getTurn());

			if (!opponent.checkToggle("double")) {
				int maydouble = 0;

				switch (board.getTurn()) {
				case 1:

					if (board.isPlayerX(opponent)) {
						maydouble = game.getBackgammonBoard().getMayDouble1();
						opponentOut = out1;
					} else {
						maydouble = game.getBackgammonBoard().getMayDouble2();
						opponentOut = out2;
					}

					break;

				case -1:

					if (board.isPlayerX(opponent)) {
						maydouble = game.getBackgammonBoard().getMayDouble1();
						opponentOut = out1;
					} else {
						maydouble = game.getBackgammonBoard().getMayDouble2();
						opponentOut = out2;
					}

					break;
				}

				if (maydouble == 1) {
					// m_roll_or_double=It's your turn. Please roll or double
					msg = jibsMessages.convert("m_roll_or_double");
					opponentOut.println(msg);
				} else {
					opponent.getClientWorker().executeCmd("roll");
				}
			} else {
				board.getpDie1().setValue(0);
				board.getpDie2().setValue(0);
				opponent.getClientWorker().executeCmd("roll");
			}
		}

		return true;
	}
}
