/******************************************************************************
 * Roll_Command - Player wants to roll the dice.
 * $Id: Roll_Command.java,v 1.2 2011/05/31 13:02:43 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Roll_Command.java,v $
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.gui.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsWriter;

/** Player wants to roll the dice.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @since 2005/03/30
 * @version $Revision: 1.2 $ <br> $Date: 2011/05/31 13:02:43 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Roll_Command.java">cvs Roll_Command.java</a>
 */
public class Roll_Command implements JibsCommand {
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		JibsWriter out = player.getOutputStream();
		JibsGame game = player.getGame();

		if (game != null) {
			BackgammonBoard board = game.getBackgammonBoard();
			Player opponent = board.getOpponent(player);
			int turn = board.getTurn();
			Player player1 = null;
			Player player2 = null;
			JibsWriter out1 = null;
			JibsWriter out2 = null;
			int nrMoves = -1;

			switch (turn) {
			case 1: // O
			player1 = board.getPlayerX();
			player2 = board.getPlayerO();
			out1 = player1.getOutputStream();
			out2 = player2.getOutputStream();
			nrMoves = game.rollO(out1, out2, player2, board, turn);

			if (nrMoves <= 0) {
				board.setTurn(-board.getTurn());
				board.getpDie1().setValue(0);
				board.getpDie2().setValue(0);
				opponent.getClientWorker().executeCmd("roll");
			} else {
				String nextCmd1 = game.checkForcedMove(player2, board,
						nrMoves, out2);
				String nextCmd2 = game.checkGreedy(player2, board, nrMoves,
						out2);
				String nextCmd = null;

				if (nextCmd1 != null) {
					nextCmd = nextCmd1;
				}

				if ((nextCmd == null) && (nextCmd2 != null)) {
					nextCmd = nextCmd2;
				}

				player2.getClientWorker().executeCmd(nextCmd);
			}

			break;

			case -1: // X
				player1 = board.getPlayerX();
				player2 = board.getPlayerO();
				out1 = player1.getOutputStream();
				out2 = player2.getOutputStream();
				nrMoves = game.rollX(out1, out2, player1, board, turn);

				if (nrMoves <= 0) {
					board.setTurn(-board.getTurn());
					board.getpDie1().setValue(0);
					board.getpDie2().setValue(0);
					opponent.getClientWorker().executeCmd("roll");
				} else {
					String nextCmd1 = game.checkForcedMove(player1, board,
							nrMoves, out1);
					String nextCmd2 = game.checkGreedy(player1, board, nrMoves,
							out1);
					String nextCmd = null;

					if (nextCmd1 != null) {
						nextCmd = nextCmd1;
					}

					if ((nextCmd == null) && (nextCmd2 != null)) {
						nextCmd = nextCmd2;
					}

					player1.getClientWorker().executeCmd(nextCmd);
				}

				break;
			}
		} else {
			// m_you_not_playing=** You're not playing.
			String msg = jibsMessages.convert("m_you_not_playing");
			out.println(msg);
		}

		return true;
	}
}
