/******************************************************************************
 * JibsTextArea.java - The console area that displays system messages.
 * $Id: JibsTextArea.java,v 1.2 2011/05/31 14:50:48 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsTextArea.java,v $
 * Revision 1.2  2011/05/31 14:50:48  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.gui;

import net.sourceforge.jibs.server.JibsServer;

import org.apache.log4j.Logger;

import java.text.DateFormat;

import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

/** The console area that displays system messages.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.2 $ <br> $Date: 2011/05/31 14:50:48 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/gui/JibsTextArea.java">cvs JibsTextArea.java</a>
 */
public class JibsTextArea extends JTextArea {
	private static final long serialVersionUID = 7806501667238028220L;
	private static Logger logger = Logger.getLogger(JibsTextArea.class);

	public JibsTextArea(JibsServer jibsServer, int r, int c) {
		super(r, c);
	}

	public static void log(JibsServer jibsServer, String s, boolean addLog) {
		java.util.Date today = new java.util.Date();
		String myDate = DateFormat.getDateInstance().format(today);
		String myClock = DateFormat.getTimeInstance().format(today);

		if (jibsServer.useSwing()) {
			jibsServer.getTextArea()
			.append(myDate + ":" + myClock + "\t" + s + "\r\n");
			jibsServer.getTextArea()
			.setCaretPosition(jibsServer.getTextArea().getText()
					.length());
		} else {
			System.out.println(s);
		}

		if (addLog) {
			logger.info(s);
		}
	}

	public static void append(JibsServer jibsServer, String s) {
		Document doc = jibsServer.getTextArea().getDocument();

		if (doc != null) {
			try {
				doc.insertString(doc.getLength(), s, null);
			} catch (BadLocationException e) {
				jibsServer.logException(e);
			}
		}
	}

	public void log(String s, boolean bWithDate) {
		if (s == null) {
			s = "null";
		}

		java.util.Date today = new java.util.Date();

		if (bWithDate) {
			String myDate = DateFormat.getDateInstance().format(today);
			String myClock = DateFormat.getTimeInstance().format(today);

			append(myDate + ":" + myClock + ":" + s + "\r\n");
			setCaretPosition(getText().length());
		} else {
			append(s + "\n");
			setCaretPosition(getText().length());
		}
	}
}
