/******************************************************************************
 * JibsServer.java - Primary jIBS window.  Contains main().
 * $Id: JibsServer.java,v 1.8 2011/06/04 01:50:56 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsServer.java,v $
 * Revision 1.8  2011/06/04 01:50:56  dick
 * Get the version via Version.getVersion().
 *
 * Revision 1.7  2011/06/03 17:06:28  dick
 * If the user data area doesn't exist, then create and populate it.
 *
 * Revision 1.6  2011/06/01 21:03:18  dick
 * Use the stored window position on startup and shutdown, if available.
 *
 * Revision 1.5  2011/05/31 17:48:49  dick
 * Fix all warnings from the import.
 *
 * Revision 1.4  2011/05/31 15:25:37  dick
 * Fix cvs link.
 *
 * Revision 1.3  2011/05/30 04:24:10  dick
 * Point the database at {user.home}/jibs/database .
 *
 * Revision 1.2  2011/05/29 05:18:54  dick
 * Use can start up, even with 0 arguments.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Writer;
import java.net.ServerSocket;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import net.sourceforge.jibs.backgammon.JibsRandom;
import net.sourceforge.jibs.command.About_Command;
import net.sourceforge.jibs.command.Accept_Command;
import net.sourceforge.jibs.command.Address_Command;
import net.sourceforge.jibs.command.Away_Command;
import net.sourceforge.jibs.command.Back_Command;
import net.sourceforge.jibs.command.Board_Command;
import net.sourceforge.jibs.command.Double_Command;
import net.sourceforge.jibs.command.Exit_Command;
import net.sourceforge.jibs.command.Invite_Command;
import net.sourceforge.jibs.command.JibsCommand;
import net.sourceforge.jibs.command.Join_Command;
import net.sourceforge.jibs.command.Kibitz_Command;
import net.sourceforge.jibs.command.Leave_Command;
import net.sourceforge.jibs.command.Login_Command;
import net.sourceforge.jibs.command.Motd_Command;
import net.sourceforge.jibs.command.Move_Command;
import net.sourceforge.jibs.command.NImplemented_Command;
import net.sourceforge.jibs.command.Password_Command;
import net.sourceforge.jibs.command.Ratings_Command;
import net.sourceforge.jibs.command.Reject_Command;
import net.sourceforge.jibs.command.Resign_Command;
import net.sourceforge.jibs.command.Roll_Command;
import net.sourceforge.jibs.command.Set_Command;
import net.sourceforge.jibs.command.Shout_Command;
import net.sourceforge.jibs.command.Shutdown_Command;
import net.sourceforge.jibs.command.Tell_Command;
import net.sourceforge.jibs.command.Toggle_Command;
import net.sourceforge.jibs.command.Unwatch_Command;
import net.sourceforge.jibs.command.Watch_Command;
import net.sourceforge.jibs.command.Wave_Command;
import net.sourceforge.jibs.command.Whisper_Command;
import net.sourceforge.jibs.command.Who_Command;
import net.sourceforge.jibs.command.Whois_Command;
import net.sourceforge.jibs.gui.ExitAction;
import net.sourceforge.jibs.gui.InfoAction;
import net.sourceforge.jibs.gui.JibsConsole;
import net.sourceforge.jibs.gui.JibsDocument;
import net.sourceforge.jibs.gui.JibsGui;
import net.sourceforge.jibs.gui.JibsMessages;
import net.sourceforge.jibs.gui.JibsStatusBar;
import net.sourceforge.jibs.gui.JibsTextArea;
import net.sourceforge.jibs.gui.JibsUserPanel;
import net.sourceforge.jibs.gui.JibsUserTableModel;
import net.sourceforge.jibs.gui.JibsWindow;
import net.sourceforge.jibs.gui.ReloadAction;
import net.sourceforge.jibs.gui.RunAction;
import net.sourceforge.jibs.gui.SplashWindowJibs;
import net.sourceforge.jibs.gui.StopAction;
import net.sourceforge.jibs.util.JibsShutdown;
import net.sourceforge.jibs.util.JibsStackTrace;
import net.sourceforge.jibs.util.JibsWriter;

import org.apache.log4j.Logger;

import com.ibatis.sqlmap.client.SqlMapClient;
import com.ibatis.sqlmap.client.SqlMapClientBuilder;
import com.jeta.forms.gui.common.FormException;

/** Primary jIBS window.  Contains main().
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.8 $ <br> $Date: 2011/06/04 01:50:56 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/server/JibsServer.java">cvs JibsServer.java</a>
 */
public class JibsServer {
	// -------------------------------------------------------------------------------------------------
	private static final Logger logger = Logger.getLogger(JibsServer.class);
	private static final long serialVersionUID = -4324503684036776752L;
	private JFrame jibsFrame;
	private String confFileName = null;
	private SqlMapClient sqlMap = null;
	private Map<String, JibsCommand> allCmds;
	private JibsDocument doc = null;
	private JibsConfiguration fConf = null;
	private JibsConsole jibsConsole = null;
	private JibsMessages jibsMessages = null;
	private JibsRandom jibsRandom;
	private JibsUserPanel jibsUserPanel = null;
	private javax.swing.JInternalFrame jInternalFrame1;
	private ServerSocket listener = null;
	private JibsWriter logWriter;
	private int portno;
	private JMenuItem startMenu;
	private JMenuItem stopMenu;
	private Thread serverThread;
	private boolean bServerRuns = false;
	private JibsGui jibsGUI = null;
	private JibsStatusBar jibsStatusbar = null;
	private InfoAction infoAction;
	private RunAction runAction;
	private StopAction stopAction;
	private ExitAction exitAction;
	private ReloadAction reloadAction;
	private JibsShutdown jibsShutdown;
	private boolean bUseSwing;

	// ------------------------------------------------------------------------------------------------------------
	private Server theServer = null;

	/** Construct the primary jibsServer.
	 * @param fileName An optional name of a .parameters file to parse
	 */
	public JibsServer(String fileName) {
		confFileName = fileName;
		fConf = new JibsConfiguration(this, fileName);
		bUseSwing = Boolean.parseBoolean(fConf.getResource("useSwing"));

		if (useSwing()) {
			int top = fConf.getIntResource("mainTop");
			int left = fConf.getIntResource("mainLeft");
			int width = fConf.getIntResource("mainWidth");
			int height = fConf.getIntResource("mainHeight");
			initComponents();

			if (width == 0) {
				Dimension dim = getJibsFrame().getToolkit().getScreenSize();
				width = (int) (dim.getWidth());
			}
			if (height == 0)
				height = 490;

			getJibsFrame().setSize(width, height);
			getJibsFrame().setLocation(left, top);
		}
	}

	/** Primary entry point into the jIBS application
	 * @param args command line args.  An optional .parameters file is the only supported arg.
	 * @throws Exception Die with any exceptions ending here in main().
	 */
	public static void main(String[] args) throws Exception {
		logger.info("Trying to start sever");
		String s = null;
		if (args.length > 0)
			s = args[0];
		else
			JibsConfiguration.maybeInitializeJibsFiles();
		JibsServer jibsServer = new JibsServer(s);

		if (jibsServer.useSwing()) {
			String v = net.sourceforge.jibs.util.Version.getVersion();
			jibsServer.getJibsFrame().setTitle("jIBS-" + v);

			URL imgURL = ClassLoader.getSystemResource("images/jibs_thumb.gif");
			ImageIcon image = new ImageIcon(imgURL);
			jibsServer.getJibsFrame().setIconImage(image.getImage());
		}

		jibsServer.startServerMenuItemActionPerformed(null);
	}

	/** Fetch the JibsCommand associated with this command string <br>
	 * get best match for strCmd to be executed, unless not ambigous
	 * re -> unknown command re (conflict (ambigous) between 'resign' and 'reject').
	 * 
	 * @param strCmd The command to query
	 * @return The matching JibsCommand or null if not found or ambiguous.
	 */
	public JibsCommand getCmd(String strCmd) {
		String sCmd = strCmd.toLowerCase();
		JibsCommand cmd = null;

		if (allCmds.containsKey(sCmd))	// exact match, return it.
			return(allCmds.get(sCmd));

		int matches = 0;
		for (String cmdString : allCmds.keySet()) {

			if (cmdString.startsWith(sCmd)) {
				cmd = (JibsCommand) allCmds.get(cmdString);
				matches++;
			}

			if (cmdString.equalsIgnoreCase(sCmd)) {
				cmd = (JibsCommand) allCmds.get(cmdString);
				matches = 0; // exact match, use it immediately

				break;
			}
		}

		if (matches > 1) {
			return null; // cmd is ambigous
		}

		return cmd;
	}

	// --------------------------------------------------------------------------------------------------------------------
	public JibsMessages getJibsMessages() {
		return jibsMessages;
	}

	public JibsRandom getJibsRandom() {
		return jibsRandom;
	}

	public JibsUserPanel getJibsUserPanel() {
		return jibsUserPanel;
	}

	public Writer getLog() {
		return logWriter;
	}

	public String getResource(String res) {
		return fConf.getResource(res);
	}

	public Server getServer() {
		return theServer;
	}

	public JTextArea getTextArea() {
		return jibsConsole.getTxtPane();
	}

	private void initComponents() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			logException(e);
		} catch (InstantiationException e) {
			logException(e);
		} catch (IllegalAccessException e) {
			logException(e);
		} catch (UnsupportedLookAndFeelException e) {
			logException(e);
		}

		jibsFrame = new JFrame();
		infoAction = new InfoAction(this, "Info",
				createImageIcon("images/info.gif", ""), "",
				KeyStroke.getKeyStroke(KeyEvent.VK_I,
						KeyEvent.ALT_MASK));
		runAction = new RunAction(this, "Start",
				createImageIcon("images/run.gif", ""), "",
				KeyStroke.getKeyStroke(KeyEvent.VK_S,
						KeyEvent.ALT_MASK));
		stopAction = new StopAction(this, "Stop",
				createImageIcon("images/stop.gif", ""), "",
				KeyStroke.getKeyStroke(KeyEvent.VK_H,
						KeyEvent.ALT_MASK));
		exitAction = new ExitAction(this, "Exit",
				createImageIcon("images/exit.gif", ""), "",
				KeyStroke.getKeyStroke(KeyEvent.VK_X,
						KeyEvent.ALT_MASK));
		reloadAction = new ReloadAction(this, "Reload",
				createImageIcon("images/refresh.gif", ""),
				"", null);

		JMenuBar jibsMenuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");

		JMenuItem exitMenu = new JMenuItem("Exit");
		exitMenu.setAction(exitAction);
		fileMenu.add(exitMenu);

		JMenu serverMenu = new JMenu("Server");
		startMenu = new JMenuItem("Start server");
		startMenu.setAction(runAction);
		stopMenu = new JMenuItem("Stop");
		stopMenu.setAction(stopAction);
		serverMenu.add(startMenu);
		serverMenu.add(stopMenu);

		JMenu aboutMenu = new JMenu("About");
		JMenuItem vmMenu = new JMenuItem("Java VM...");
		vmMenu.setAction(infoAction);
		aboutMenu.add(vmMenu);
		jibsMenuBar.add(fileMenu);
		jibsMenuBar.add(serverMenu);
		jibsMenuBar.add(aboutMenu);
		try {
			jibsGUI = new JibsGui(this);
		} catch (FormException e) {
			e.printStackTrace();
			System.exit(99);
		}

		jInternalFrame1 = new javax.swing.JInternalFrame();

		jibsConsole = new JibsConsole(this);
		jibsStatusbar = new JibsStatusBar(this);
		jibsUserPanel = new JibsUserPanel(this);
		jibsGUI.getBtnInfo().setAction(infoAction);
		jibsGUI.getBtnExit().setAction(exitAction);
		jibsGUI.getBtnStart().setAction(runAction);
		jibsGUI.getBtnStop().setAction(stopAction);
		jibsGUI.getBtnReload().setAction(reloadAction);

		jInternalFrame1.setVisible(true);

		getJibsFrame().addWindowListener(new JibsWindow(this));

		getJibsFrame().getContentPane()
		.add(jibsMenuBar, java.awt.BorderLayout.NORTH);
		getJibsFrame().getContentPane()
		.add(jibsGUI, java.awt.BorderLayout.CENTER);
	}

	/** Do any cleanup prior to final shutdown.  Currently this means to save the window position.
	 */
	public void shutdown() {
		if (useSwing()) {
			Dimension dim = getJibsFrame().getSize();
			Point p = getJibsFrame().getLocation();
			fConf.writeWindowPosition(p.y, p.x, dim.width, dim.height);
		}
	}

	/** Did the client type one of the eight exit commands?
	 * @param cmd The command to check
	 * @return true if the command is bye, adios, ciao, etc.
	 */
	public static boolean isExitCmd(String cmd) {
		String[] allExitCmds = {
				"bye", "adios", "ciao", "end",
				"exit",	"logout", "quit", "tschoe"
		};
		boolean b = false;

		for (int i = 0; i < allExitCmds.length; i++)
			if (allExitCmds[i].equalsIgnoreCase(cmd)) {
				b = true;
			}

		return b;
	}

	public void startServerMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
		SplashWindowJibs splashWindow = null;

		try {
			if (evt == null) {
				if (useSwing()) {
					String splashScreen = fConf.getResource("splashScreen");
					splashWindow = new SplashWindowJibs(this, splashScreen, null);
				}
			}

			jibsMessages = new JibsMessages(this,
					fConf.getResource("MessageFile"));

			portno = Integer.parseInt(fConf.getResource("Port"));
			listener = new ServerSocket(portno);
			allCmds = new HashMap<String, JibsCommand>();

			// register all commands
			allCmds.put("about", new About_Command());
			allCmds.put("accept", new Accept_Command());
			allCmds.put("address", new Address_Command());
			allCmds.put("average", new NImplemented_Command());
			allCmds.put("away", new Away_Command());
			allCmds.put("back", new Back_Command());
			allCmds.put("beaver", new NImplemented_Command());
			allCmds.put("blind", new NImplemented_Command());
			allCmds.put("board", new Board_Command());
			allCmds.put("bye", new Exit_Command());
			allCmds.put("adios", new Exit_Command());
			allCmds.put("ciao", new Exit_Command());
			allCmds.put("end", new Exit_Command());
			allCmds.put("exit", new Exit_Command());
			allCmds.put("logout", new Exit_Command());
			allCmds.put("quit", new Exit_Command());
			allCmds.put("tschoe", new Exit_Command());
			allCmds.put("clear", new NImplemented_Command());
			allCmds.put("cls", new NImplemented_Command());
			allCmds.put("date", new NImplemented_Command());
			allCmds.put("dicetest", new NImplemented_Command());
			allCmds.put("double", new Double_Command());
			allCmds.put("erase", new NImplemented_Command());
			allCmds.put("gag", new NImplemented_Command());
			allCmds.put("help", new NImplemented_Command());
			allCmds.put("invite", new Invite_Command());
			allCmds.put("join", new Join_Command());
			allCmds.put("kibitz", new Kibitz_Command());
			allCmds.put("last", new NImplemented_Command());
			allCmds.put("leave", new Leave_Command());
			allCmds.put("login", new Login_Command());
			allCmds.put("look", new NImplemented_Command());
			allCmds.put("matrix", new NImplemented_Command());
			allCmds.put("man", new NImplemented_Command());
			allCmds.put("message", new NImplemented_Command());
			allCmds.put("motd", new Motd_Command());
			allCmds.put("move", new Move_Command());
			allCmds.put("m", new Move_Command());
			allCmds.put("off", new NImplemented_Command());
			allCmds.put("oldboard", new NImplemented_Command());
			allCmds.put("oldmoves", new NImplemented_Command());
			allCmds.put("otter", new NImplemented_Command());
			allCmds.put("panic", new NImplemented_Command());
			allCmds.put("password", new Password_Command());
			allCmds.put("pip", new NImplemented_Command());
			allCmds.put("raccoon", new NImplemented_Command());
			allCmds.put("ratings", new Ratings_Command());
			allCmds.put("rawwho", new NImplemented_Command());
			allCmds.put("redouble", new NImplemented_Command());
			allCmds.put("reject", new Reject_Command());
			allCmds.put("resign", new Resign_Command());
			allCmds.put("roll", new Roll_Command());
			allCmds.put("save", new NImplemented_Command());
			allCmds.put("say", new NImplemented_Command());
			allCmds.put("set", new Set_Command());
			allCmds.put("shout", new Shout_Command());
			allCmds.put("show", new NImplemented_Command());
			allCmds.put("shutdown", new Shutdown_Command());
			allCmds.put("sortwho", new NImplemented_Command());
			allCmds.put("stat", new NImplemented_Command());
			allCmds.put("tell", new Tell_Command());
			allCmds.put("time", new NImplemented_Command());
			allCmds.put("toggle", new Toggle_Command());
			allCmds.put("unwatch", new Unwatch_Command());
			allCmds.put("version", new NImplemented_Command());
			allCmds.put("watch", new Watch_Command());
			allCmds.put("waitfor", new NImplemented_Command());
			allCmds.put("wave", new Wave_Command());
			allCmds.put("where", new NImplemented_Command());
			allCmds.put("whisper", new Whisper_Command());
			allCmds.put("who", new Who_Command());
			allCmds.put("whois", new Whois_Command());

			if (useSwing()) {
				jibsUserPanel.readAllPlayers();
			}

			jibsRandom = new JibsRandom();
			theServer = new Server(jibsMessages, this, listener, portno);

			serverThread = new Thread(theServer);

			serverThread.start();

			if (useSwing()) {
				if (splashWindow != null) {
					splashWindow.dispose();
				}

				runAction.setEnabled(false);
				stopAction.setEnabled(true);
				bServerRuns = true;
				getJibsFrame().setVisible(true); // now display the main window
			}

			JibsTextArea.log(this, "Server started on port " + portno, true);
		} catch (Exception e) {
			String sMsg = "jIBS could not start. Check the configuration file '" +
			confFileName + "'";
			logger.fatal(sMsg, e);

			if (splashWindow != null) {
				splashWindow.dispose();
			}

			JOptionPane.showMessageDialog(null, sMsg, "Warning",
					JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		}
	}

	public void stopServerMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
		if (runs()) {
			boolean bStop = false;

			try {
				Server server = getServer();
				int onlinePlayer = server.getAllClients().size();

				if (evt != null) {
					if (onlinePlayer > 0) {
						String warnExit = getResource("warnExit");
						Boolean bWarn = Boolean.valueOf(warnExit);

						if (bWarn) {
							String msg;

							if (onlinePlayer > 1) {
								msg = "There are still " + onlinePlayer +
								" players online.\n" +
								"When you continue these players will be disconnected without warning.\n" +
								"They might consider this to be rude behaviour.\n" +
								"\n" + "Do you want to continue?";
							} else {
								msg = "There is still one player online.\n" +
								"When you continue this player will be disconnected without warning.\n" +
								"He/She might consider this to be rude behaviour.\n" +
								"\n" + "Do you want to continue?";
							}

							if (useSwing()) {
								int option = JOptionPane.showConfirmDialog(getJibsFrame(),
										msg,
										"Exit",
										JOptionPane.YES_NO_OPTION);

								if (option == JOptionPane.YES_OPTION) {
									bStop = true;
								}
							} else {
								bStop = true;
							}
						} else {
							bStop = true;
						}
					} else {
						bStop = true;
					}
				} else {
					bStop = true;
				}

				if (bStop) {
					server.getListener().close();
					server.setRuns(false);
					getServerThread().join();
					server.closeAllClients();
					bServerRuns = false;
				}
			} catch (IOException e) {
				logException(e);
			} catch (InterruptedException e) {
				logException(e);
			}

			runAction.setEnabled(true);
			stopAction.setEnabled(false);
			JibsTextArea.log(this, "Server stopped.", true);
		}
	}

	public void logException(Exception e) {
		String e1 = JibsStackTrace.getCustomStackTrace(e);
		JibsTextArea.log(this, e1, true);
		logger.warn(e);
	}

	public SqlMapClient getSqlMapInstance() {
		if (sqlMap == null) {
			String sqlMapConfig = "database/SqlMapConfig.xml";
			if (fConf.hasProperty("dbConfiguration"))
				sqlMapConfig = fConf.getResource("dbConfiguration");		// Points to SqlMapConfig.xml
			InputStream is = ClassLoader.getSystemResourceAsStream(sqlMapConfig);
			BufferedReader buffer = new BufferedReader(new InputStreamReader(is));
			//Reader r = Resources.
			//Reader reader = Resources.getResourceAsReader(is);
			Properties props = new Properties();
			props.setProperty("home", System.getProperty("user.home"));
			String s = "jdbc:hsqldb:file:{user.home}/jibs/database/jibs";
			s = JibsConfiguration.unwindProperties(s);
			props.setProperty("home", s);
			//				sqlMap = SqlMapClientBuilder.buildSqlMapClient(buffer, props);
			sqlMap = SqlMapClientBuilder.buildSqlMapClient(buffer, System.getProperties());
		}

		return sqlMap;
	}

	public Thread getServerThread() {
		return serverThread;
	}

	public void setServerThread(Thread serverThread) {
		this.serverThread = serverThread;
	}

	public boolean runs() {
		return bServerRuns;
	}

	public JibsStatusBar getStatusBar() {
		return jibsStatusbar;
	}

	public JibsGui getJibsGUI() {
		return jibsGUI;
	}

	public JibsUserTableModel getUserTableModel() {
		return jibsGUI.getUserTableModel();
	}

	public JibsShutdown getJibsShutdown() {
		return jibsShutdown;
	}

	public void setJibsShutdown(JibsShutdown jibsShutdown) {
		this.jibsShutdown = jibsShutdown;
	}

	public JibsDocument getDoc() {
		return doc;
	}

	public void setDoc(JibsDocument doc) {
		this.doc = doc;
	}

	protected static ImageIcon createImageIcon(String path, String description) {
		URL imgURL = ClassLoader.getSystemResource(path);

		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			logger.warn("Couldn't find file: " + path);

			return null;
		}
	}

	public JFrame getJibsFrame() {
		return jibsFrame;
	}

	public boolean useSwing() {
		return bUseSwing;
	}
}
